{-
   BezMesh.hs  (adapted from bezmesh.c which is (c) Silicon Graphics, Inc)
   This file is part of HOpenGL - a binding of OpenGL and GLUT for Haskell.
   Copyright (C) 2000  Sven Panne <Sven.Panne@informatik.uni-muenchen.de>

   This program renders a lighted, filled Bezier surface,
   using two-dimensional evaluators.
-}

import System.Exit ( exitWith, ExitCode(ExitSuccess) )

import GL
import Graphics.UI.GLUT

ctrlPoints :: [[Vertex3 GLfloat]]
ctrlPoints = [
   [ Vertex3 (-1.5) (-1.5)   4.0,  Vertex3 (-0.5) (-1.5)   2.0,
     Vertex3   0.5  (-1.5) (-1.0), Vertex3   1.5  (-1.5)   2.0 ],
   [ Vertex3 (-1.5) (-0.5)   1.0,  Vertex3 (-0.5) (-0.5)   3.0,
     Vertex3   0.5  (-0.5)   0.0,  Vertex3   1.5  (-0.5) (-1.0) ],
   [ Vertex3 (-1.5)   0.5    4.0,  Vertex3 (-0.5)   0.5    0.0,
     Vertex3   0.5    0.5    3.0,  Vertex3   1.5    0.5    4.0 ],
   [ Vertex3 (-1.5)   1.5  (-2.0), Vertex3 (-0.5)   1.5  (-2.0),
     Vertex3   0.5    1.5    0.0,  Vertex3   1.5    1.5  (-1.0) ]]

initlights :: IO ()
initlights = do
   enable Lighting
   enable (Light 0)

   mapM_ (light (Light 0)) [ LightColor Ambient (Color4 0.2 0.2 0.2 1.0),
                             Position (Vertex4 0.0 0.0 2.0 1.0) ]

   mapM_ (material Front) [ MaterialColor Diffuse  (Color4 0.6 0.6 0.6 1.0),
                            MaterialColor Specular (Color4 1.0 1.0 1.0 1.0),
                            Shininess 50.0 ]

display :: DisplayCallback
display = do
   clear [ ColorBufferBit, DepthBufferBit ]
   pushMatrix
   rotate (85.0 :: GLfloat) (Vector3 1.0 1.0 1.0)
   evalMesh2 Fill 0 20 0 20
   popMatrix
   flush

myInit :: IO ()
myInit = do
   clearColor (Color4 0.0 0.0 0.0 0.0)
   enable DepthTest
   withArray (concat ctrlPoints) $
      map2 Map2Vertex3 (0 :: GLfloat) 1 3 4 0 1 12 4 . castPtr
   enable Map2Vertex3
   enable AutoNormal
   mapGrid2 20 (0.0 :: GLfloat) 1.0 20 0.0 1.0
   initlights  -- for lighted version only

reshape :: ReshapeCallback
reshape (WindowSize w h) = do
   viewport (0, 0) (fromIntegral w, fromIntegral h)
   matrixMode Projection
   loadIdentity
   let wf = fromIntegral w
       hf = fromIntegral h
   if (w <= h)
      then ortho (-4.0) 4.0 (-4.0*hf/wf) (4.0*hf/wf) (-4.0) 4.0
      else ortho (-4.0*wf/hf) (4.0*wf/hf) (-4.0) 4.0 (-4.0) 4.0
   matrixMode Modelview
   loadIdentity

keyboard :: KeyboardMouseCallback
keyboard (Char '\27') Down _ _ = exitWith ExitSuccess
keyboard _            _    _ _ = return ()

main :: IO ()
main = do
   (progName, _args) <- getArgsAndInitialize
   setInitialDisplayMode [ Single, RGB, Graphics.UI.GLUT.Depth ]
   setInitialWindowSize (WindowSize 500 500)
   setInitialWindowPosition (WindowPosition 100 100)
   createWindow progName
   myInit
   setReshapeCallback  (Just reshape)
   setDisplayCallback display
   setKeyboardMouseCallback (Just keyboard)
   mainLoop
